using Adaptation._Tests.Shared;
using Microsoft.Extensions.Logging;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Text;

namespace Adaptation._Tests.Static;

[TestClass]
public class RsM : LoggingUnitTesting, IDisposable
{

#pragma warning disable CA2254
#pragma warning disable IDE0060

    internal static RsM LoggingUnitTesting { get; private set; }

    public RsM() : base(testContext: null, declaringType: null)
    {
        if (LoggingUnitTesting is null)
            throw new Exception();
    }

    public RsM(TestContext testContext) : base(testContext, new StackFrame().GetMethod().DeclaringType)
    {
    }

    [ClassInitialize]
    public static void ClassInitialize(TestContext testContext) => LoggingUnitTesting ??= new RsM(testContext);

    [ClassCleanup()]
    public static void ClassCleanup()
    {
        LoggingUnitTesting?.Logger?.LogInformation("Cleanup");
        LoggingUnitTesting?.Dispose();
    }

    private static void NonThrowTryCatch()
    {
        try
        { throw new Exception(); }
        catch (Exception) { }
    }

    [TestMethod]
    public void TestDateTime()
    {
        DateTime dateTime = DateTime.Now;
        Assert.IsTrue(dateTime.ToString("M/d/yyyy h:mm:ss tt") == dateTime.ToString());
    }

    [TestMethod]
    public void TestDescriptor()
    {
        FileHandlers.RsM.Descriptor descriptor;
        MethodBase methodBase = new StackFrame().GetMethod();
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Getting configuration"));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor(string.Empty);
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("12-123456-1234");
        Assert.IsTrue(descriptor.Reactor is "12");
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(descriptor.PSN is "1234");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("123456");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("1T123456");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("MP");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(descriptor.Employee is "MP");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("12-123456-1234.2-1");
        Assert.IsTrue(descriptor.Reactor is "12");
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(descriptor.PSN is "1234");
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("12-123456-1234.02-01");
        Assert.IsTrue(descriptor.Reactor is "12");
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(descriptor.PSN is "1234");
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("12-123456-1234.02-1");
        Assert.IsTrue(descriptor.Reactor is "12");
        Assert.IsTrue(descriptor.RDS is "123456");
        Assert.IsTrue(descriptor.PSN is "1234");
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("20");
        Assert.IsTrue(descriptor.Reactor is "20");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("20.2");
        Assert.IsTrue(descriptor.Reactor is "20");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("20.2.1");
        Assert.IsTrue(descriptor.Layer is "2");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(descriptor.Reactor is "20");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("20.1.1");
        Assert.IsTrue(descriptor.Layer is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(descriptor.Reactor is "20");
        Assert.IsTrue(descriptor.Zone is "1");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("P2-LOW-RR");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(descriptor.PSN is "RR");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(descriptor.Reactor is "P2");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("75-QP1414-SPLIT4");
        Assert.IsTrue(!string.IsNullOrEmpty(descriptor.Run));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(descriptor.PSN is "SPLIT4");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.RDS));
        Assert.IsTrue(descriptor.Reactor is "75");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("B48");
        Assert.IsTrue(descriptor.Run == "B48");
        descriptor = FileHandlers.RsM.ProcessData.GetDescriptor("B48");
        Assert.IsTrue(descriptor.Run == "B48");
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Layer));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.PSN));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Reactor));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Zone));
        Assert.IsTrue(string.IsNullOrEmpty(descriptor.Employee));
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Exit"));
        NonThrowTryCatch();
    }

#if DEBUG
    [Ignore]
#endif
    [TestMethod]
    public void Production()
    {
        MethodBase methodBase = new StackFrame().GetMethod();
        StringBuilder results = new();
        (string cellInstanceName, string cellInstanceVersionName)[] collection = new (string, string)[]
        {
            new("CDE3", "v2.57.0"),
            new("CDE4", "v2.57.0"),
            new("CDE5", "v2.57.0"),
            new("CDE6", "v2.57.0"),
            new("CDE3-EQPT", "v2.57.0"),
            new("CDE4-EQPT", "v2.57.0"),
            new("CDE5-EQPT", "v2.57.0"),
            new("CDE6-EQPT", "v2.57.0"),
        };
        string production = "http://messa08ec.infineon.com:9003/CellInstanceServiceV2";
        Shared.PasteSpecialXml.EAF.XML.API.CellInstance.CellInstanceVersion cellInstanceVersion;
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Getting configuration"));
        foreach ((string cellInstanceName, string cellInstanceVersionName) in collection)
        {
            cellInstanceVersion = AdaptationTesting.GetCellInstanceVersion($"{production}/{cellInstanceName}/{cellInstanceVersionName}/configuration");
            _ = results.AppendLine($"{cellInstanceName}\t{cellInstanceVersionName}\t{cellInstanceVersion.EdaConnection.PortNumber}");
        }
        string sourceDirectory = "D:/Tmp/cellInstanceVersion.EdaConnection.PortNumber";
        if (!Directory.Exists(sourceDirectory))
            _ = Directory.CreateDirectory(sourceDirectory);
        File.WriteAllText(Path.Combine(sourceDirectory, $"{methodBase.Module.Name}-{methodBase.ReflectedType.Name}-{methodBase.Name}.tsv"), results.ToString());
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Exit"));
    }

    [TestMethod]
    public void CellInstanceVersionEdaConnectionPortNumber()
    {
        MethodBase methodBase = new StackFrame().GetMethod();
        string sourceDirectory = "D:/Tmp/cellInstanceVersion.EdaConnection.PortNumber";
        if (Directory.Exists(sourceDirectory))
        {
            string destinationDirectory = Path.Combine(sourceDirectory, "All");
            if (!Directory.Exists(destinationDirectory))
                _ = Directory.CreateDirectory(destinationDirectory);
            List<string> lines = new();
            string[] files = Directory.GetFiles(sourceDirectory, "*.tsv", SearchOption.TopDirectoryOnly);
            foreach (string file in files)
                lines.AddRange(File.ReadAllLines(file));
            File.WriteAllLines(Path.Combine(destinationDirectory, $"{DateTime.Now.Ticks}.tsv"), lines.Distinct().OrderBy(l => l));
        }
        LoggingUnitTesting.Logger.LogInformation(string.Concat(methodBase.Name, " - Exit"));
        NonThrowTryCatch();
    }

}